/*

                             C S V R E A D
                             =============

              Read Hacker's Diet Eat Watch CSV database.

                            by John Walker
                       http://www.fourmilab.ch/
                 This program is in the public domain

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef WIN32
#include "getopt.h"
#else
#include <unistd.h>
#endif

#include "astro.h"
#include "database.h"
#include "csvin.h"
#include "memdb.h"
#include "version.h"

static char *weightUnits[] = {
    "Kilograms",
    "Pounds",
    "Stones"
};

static double dOffset;

/*  dumpMonth  --  Dump month as primate-readable text.  */

static char *weekdays[] = {
                            "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
                          },

            *months[] =   {
                            "January", "February", "March",
                            "April", "May", "June",
                            "July", "August", "September",
                            "October", "November", "December"
                          };

static void dumpMonth(FILE *fo, struct hdMonth *m)
{
    int i;
    char srung[4], swgt[6];
    long zdo = (long) (m->lastModificationTime - dOffset);

    fprintf(fo, "\n%4d-%02d [%s]  (%d days, first day %d, start trend = %.2f %s)\n",
            m->year, m->month + 1, months[m->month], m->days, m->firstDay,
            m->trendCarryForward / 10000.0, weightUnits[m->weightUnit]);


            fprintf(fo, "    Last modified: %s",
                asctime(gmtime(&zdo)));
    fprintf(fo, " Date  Weight Rung Flg  Comment\n");
    for (i = 0; i < ((int) m->days); i++) {
        srung[0] = swgt[0] = 0;

        if (m->rungArray[i] != RUNG_ABSENT) {
            sprintf(srung, "%2d", m->rungArray[i]);
        }
        if (m->weightArray[i] != 0) {
            sprintf(swgt, "%5.1f", m->weightArray[i] / 100.0);
        }
        fprintf(fo, "%2d %s  %5s  %2s   %c   %s\n",
            i + 1, weekdays[(m->firstDay + i) % 7],
            swgt, srung,
            m->flagsArray[i] ? 'X' : ' ',
            m->commentArray[i] == NULL ? "" : m->commentArray[i]);
    }
}

static struct hdMonth *currentMonth = NULL;
static Boolean headerChanged = false;
static trend hTrend = 0;
static unsigned char hWeightUnit = 0;
static unsigned long hLastModTime = 0;

/*  addDay  --  Add a daily log entry to the in-memory database.
                Note that we are called with a month number from
                0 to 11 but a civil day number of 1 to 31.  This
                is in keeping with the Unix ctime() convention
                used elsewhere in these programs.  */

static void addDay(int year, int month, int day,
                   weight wgt,
                   rung rng,
                   Boolean flag,
                   char *comment)
{
    if ((currentMonth == NULL) ||
        (currentMonth->year != year) ||
        (currentMonth->month != month)) {
        currentMonth = mdGetMonth(year, month);
    }

    if (currentMonth != NULL) {
        if (headerChanged) {
            currentMonth->trendCarryForward = hTrend;
            currentMonth->weightUnit = hWeightUnit;
            currentMonth->lastModificationTime = hLastModTime;
            headerChanged = false;
        }
        if (wgt != 0) {
            currentMonth->weightArray[day - 1] = wgt;
        }
        if (rng != RUNG_ABSENT) {
            currentMonth->rungArray[day - 1] = rng;
        }
        currentMonth->flagsArray[day - 1] = flag;
        if (comment[0] != 0) {
            currentMonth->commentArray[day - 1] = strdup(comment);
        }
    }
}

/*  USAGE  --  Print how-to-call information.  */

static void usage(void)
{
    fprintf(stderr, "csvread [options] [ifile] [ofile]\n");
    fprintf(stderr, "Dump Hacker's Diet Eat Watch CSV file.\n");
    fprintf(stderr, "Release %s.\n", Version);
    fprintf(stderr, "   Options:\n");
    fprintf(stderr, "       -u             Print this message\n");
    fprintf(stderr, "by John Walker (http://www.fourmilab.ch/)\n");
    fprintf(stderr, "This program is in the public domain.\n");
}

/*  Main program.  */

int main(int argc, char *argv[])
{
    extern char *optarg;
    extern int optind;

    int i, opt, dump = 0, new = 1, verbose = 0;
    struct hdMonth *m;
    FILE *fi = stdin, *fo = stdout;
    char s[132];

    dOffset = ((ucttoj(1970, 0, 1, 0, 0, 0) + 0.001) -
                ucttoj(1904, 0, 1, 0, 0, 0)
              );
    dOffset *= 24L * 60L * 60;

    while ((opt = getopt(argc, argv, "u")) != -1) {
        switch (opt) {
            case 'u':
                usage();
                return 0;

            case '?':
                usage();
                return 2;
        }
    }

    i = 0;

    for (; optind < argc; optind++) {
        switch (i) {
            case 0:
                fi = fopen(argv[optind], "r");
                if (fi == NULL) {
                    fprintf(stderr, "Cannot open input file %s.\n", argv[optind]);
                    return 2;
                }
                break;

            case 1:
                fo = fopen(argv[optind], "w");
                if (fo == NULL) {
                    fprintf(stderr, "Cannot create output file %s.\n", argv[optind]);
                    return 2;
                }
                break;
        }
        i++;
    }

    while (fgets(s, sizeof s, fi) != NULL) {
        int year, month, day;
        weight wgt;
        rung rng;
        Boolean flag;
        char comment[132];

        if (parseDay(s, &year, &month, &day, &wgt, &rng, &flag, comment) ||
            parseDayOld(s, &year, &month, &day, &wgt, &rng, &flag, comment)) {
            addDay(year, month - 1, day, wgt, rng, flag, comment);
        } else if (parseHeader(s, &hTrend, &hWeightUnit, &hLastModTime)) {
            headerChanged = true;
        }
    }

    fclose(fi);

    fprintf(fo, "Database index:\n");
    mdDumpDatabase(fo);
    fprintf(fo, "\nRecord dump:\n");
    i = 0;
    while ((m = mdIndexMonth(i)) != NULL) {
        dumpMonth(fo, m);
        i++;
    }
    mdEmptyDatabase();

    fclose(fo);

    return 0;
}
