/*

                    In-memory monthly log database

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
#define TRACE
*/

#ifdef TRACE
#include <assert.h>
#endif

#include "database.h"
#include "memdb.h"

static struct hdMonth **mtable = NULL;
static int nMonths = 0, nTable = 0, findIndex = -1;

#define mExpand   12                  /* Month table expansion interval */

/*  mdIndexMonth  --  Return month from database by record index.  */

struct hdMonth *mdIndexMonth(int index)
{
    return ((index < 0) || (index >= nMonths)) ? NULL : mtable[index];
}

/*  mdFindMonth  --  Find month in database.  The index of the
                     last month found is placed in the local
                     variable findIndex.  */

struct hdMonth *mdFindMonth(int year, int month)
{
    int i;
    struct hdMonth *found = NULL;

    findIndex = -1;
    for (i = 0; i < nMonths; i++) {
        if ((mtable[i]->year == year) && (mtable[i]->month == month)) {
            found = mtable[i];
            findIndex = i;
            break;
        }
    }

#ifdef TRACE
printf("mdFindMonth(%d, %d) = %ld, findIndex = %d\n", year, month, found, findIndex);
#endif
    return found;
}

/*  mdPutMonth  --  Add or replace month entry in database.  */

void mdPutMonth(struct hdMonth *m)
{
    int i, j, year = m->year, month = m->month;
    struct hdMonth *om = mdFindMonth(year, month);

    if (om != NULL) {
        disposeMonth(om);
        mtable[findIndex] = m;
    } else {
        if (mtable == NULL) {
            nTable = mExpand;
            mtable = malloc(nTable * sizeof(struct hdMonth *));
#ifdef TRACE
            fprintf(stderr, "Created table: nTable = %d nMonths = %d\n", nTable, nMonths);
#endif
        }
        if (nMonths >= nTable) {
            struct hdMonth **ntable = malloc((nTable + mExpand) * sizeof(struct hdMonth *));

            memcpy(ntable, mtable, nTable * sizeof(struct hdMonth *));
            free(mtable);
            mtable = ntable;
            nTable += mExpand;
#ifdef TRACE
            fprintf(stderr, "Expanded table: nTable = %d nMonths = %d\n", nTable, nMonths);
#endif
        }

        /* Find position in table before which this item should be added. */

        if (mtable != NULL) {
            for (i = 0; i < nMonths; i++) {
                if ((mtable[i]->year > year) ||
                    ((mtable[i]->year == year) && (mtable[i]->month > month))) {
                    break;
                }
            }

#ifdef TRACE
            assert(i <= nMonths);
            if (i < nMonths) {
                fprintf(stderr, "Add new %04d-%02d before record %d of %d: %04d-%02d\n", year, month, i, nMonths, mtable[i]->year, mtable[i]->month);
            } else {
                fprintf(stderr, "Add new %04d-%02d at end of %d record database\n", year, month, nMonths);
            }
#endif

            /* If records exist after the insert location, shift
               existing records down.  Note that we've already
               guaranteed that there's room. */

            if (i < nMonths) {
                for (j = nMonths; j > i; j--) {
                    mtable[j] = mtable[j - 1];
                }
            }
            nMonths++;
            mtable[i] = m;
        }
    }
}

/*  mdGetMonth  --  Get month from database, creating if required.  */

struct hdMonth *mdGetMonth(int year, int month)
{
    struct hdMonth *m = mdFindMonth(year, month);

    if (m == NULL) {
        m = newMonth(year, month);
        if (m != NULL) {
            mdPutMonth(m);
        }
    }

    return m;
}

/*  mdEmptyDatabase  --  Empty the database and release all associated
                         storage.  The database is returned to the
                         original state and may be re-used subsequently.
                         It's OK to call this even if the database is
                         presently empty.  */

void mdEmptyDatabase(void)
{
    if (mtable != NULL) {
        int i;

        for (i = 0; i < nMonths; i++) {
            disposeMonth(mtable[i]);
        }
        free(mtable);
        mtable = NULL;
    }
    nMonths = 0;
    nTable = 0;
    findIndex = -1;
}

/*  mdDumpDatabase  --  Print summary of records in database.  */

void mdDumpDatabase(FILE *fo)
{
    int i;

    for (i = 0; i < nMonths; i++) {
        fprintf(fo, "Record %d: %04d-%02d\n", i, mtable[i]->year, mtable[i]->month);
    }
}
